package com.sf.vertx.handle;

import com.sf.vertx.api.pojo.AppCurrentLimitingConfig;
import com.sf.vertx.init.DynamicBuildServer;
import com.sf.vertx.service.impl.AppConfigServiceImpl;

import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.HttpException;
import lombok.extern.slf4j.Slf4j;

/***
 * redis存储
 * 
 * @author xy
 *
 */
@Slf4j
public class RateLimitHandlerRedisImpl implements RateLimitHandler {
	private RedisRateLimiter rateLimiter;

	public RateLimitHandlerRedisImpl(RedisRateLimiter rateLimiter) {
		this.rateLimiter = rateLimiter;
	}

	@Override
	public void handle(RoutingContext rc) {
		String sacAppHeaderKey = rc.request().headers().get(DynamicBuildServer.SAC_APP_HEADER_KEY);
		// TODO 判断是否开启限流配置
		log.info("RateLimitHandlerRedisImpl request:{}", sacAppHeaderKey);
//		rc.next();
//		return;
		// 获取模式
		int pattern = 1; // 1:app,2:接口默认,3:服务接口配置限流策略
		if (rateLimiter.acquire(rc, pattern)) {
			log.info("rateLimiter.acquire true");
			rc.next();
			return;
		} else {
			switch (pattern) {
			case 1:
				AppCurrentLimitingConfig appCurrentLimitingConfig = AppConfigServiceImpl
						.getAppCurrentLimitingConfig(sacAppHeaderKey);
				rc.fail(new HttpException(429, appCurrentLimitingConfig.getDefaultResponse()));
				return;
//				JsonObject dataJson = new JsonObject(appCurrentLimitingConfig.getDefaultResponse());
//				rc.response().setChunked(true).setStatusCode(429).putHeader("Content-Type", "application/json").end(dataJson.toBuffer());
//				return;
			}
		}
	}
}
