package com.sf.vertx.service.impl;

import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.sf.vertx.api.pojo.AppConfig;
import com.sf.vertx.api.pojo.AppCurrentLimitingConfig;
import com.sf.vertx.api.pojo.VertxConfig;
import com.sf.vertx.constans.RedisKeyConfig;
import com.sf.vertx.service.AppConfigService;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class AppConfigServiceImpl implements AppConfigService {
	@Value("${server.vertx.environment}")
	private String vertxEnvironment;
	@Autowired
	private RedisTemplate<String, String> redisTemplate;
	private static final ConcurrentHashMap<String, AppConfig> CACHE_APP_CONFIG = new ConcurrentHashMap<>();
	private static final ConcurrentHashMap<String, AppCurrentLimitingConfig> CACHE_APP_CURRENT_LIMITING_CONFIG = new ConcurrentHashMap<>();

	public static boolean appDataSecurity(String appCode) {
		return CACHE_APP_CONFIG.get(appCode) != null && CACHE_APP_CONFIG.get(appCode).getDataSecurity() != null ? true
				: false;
	}

	public static AppCurrentLimitingConfig getAppCurrentLimitingConfig(String appCode) {
		return CACHE_APP_CURRENT_LIMITING_CONFIG.get(appCode);
	}

	/***
	 * 从redis加载数据
	 */
	public ConcurrentHashMap<String, AppConfig> loadAllConfig() {
		Set<String> set = redisTemplate.opsForZSet().range(RedisKeyConfig.APP_CONFIG_SET_KEY, 0, -1);
		for (String appCode : set) {
			String appCodeKey = RedisKeyConfig.APP_CONFIG_PREFIX_KEY + ":" + appCode;
			String appCodeValue = redisTemplate.opsForValue().get(appCodeKey);
			if (StringUtils.isNotBlank(appCodeValue)) {
				AppConfig appConfig = JSON.parseObject(appCodeValue, new TypeReference<AppConfig>() {
				});
				CACHE_APP_CONFIG.put(appCode, appConfig);

			}
		}

		// TODO 限流
		AppCurrentLimitingConfig appCurrentLimitingConfig = new AppCurrentLimitingConfig();
		appCurrentLimitingConfig.setThreshold(1);
		appCurrentLimitingConfig.setTimeWindow(10);
		appCurrentLimitingConfig.setDefaultResponse(
				"{\n" + "    \"msg\": \"接口繁忙请重试\",\n" + "    \"code\": 501,\n" + "    \"data\": \"到达限流阈值\"\n" + "}");
		CACHE_APP_CURRENT_LIMITING_CONFIG.put("dsafdsfadafhappd", appCurrentLimitingConfig);
		CACHE_APP_CURRENT_LIMITING_CONFIG.put("dsafdsfadafhappC", appCurrentLimitingConfig);
		log.info("cacheAppConfig:{}", JSON.toJSONString(CACHE_APP_CONFIG));

		return CACHE_APP_CONFIG;
	}

	public AppConfig getAppConfig(String appCode) {
		String appCodeKey = RedisKeyConfig.APP_CONFIG_PREFIX_KEY + ":" + appCode;
		String appCodeValue = redisTemplate.opsForValue().get(appCodeKey);
		if (StringUtils.isNotBlank(appCodeValue)) {
			AppConfig appConfig = JSONObject.parseObject(appCodeValue, AppConfig.class);
			return appConfig;
		}
		return null;
	}

	/***
	 * 新增、修改
	 * 
	 * @param appConfig
	 */
	public void addAppConfig(String appConfigStr) {
		AppConfig appConfig = JSON.parseObject(appConfigStr, AppConfig.class);
		redisTemplate.opsForZSet().add(RedisKeyConfig.APP_CONFIG_SET_KEY, appConfig.getAppCode(), 0);
		String appCodeKey = RedisKeyConfig.APP_CONFIG_PREFIX_KEY + ":" + appConfig.getAppCode();
		redisTemplate.opsForValue().set(appCodeKey, appConfigStr);

		// 发送redis队列,vertx处理
		// String queue = RedisKeyConfig.VETX_ENVIRONMENT_KEY+vertxEnvironment+":list";

	}

	/***
	 * 删除
	 * 
	 * @param appConfig
	 */
	public void deleteAppConfig(AppConfig appConfig) {
		redisTemplate.opsForZSet().remove(RedisKeyConfig.APP_CONFIG_SET_KEY, appConfig.getAppCode());
		String appCodeKey = RedisKeyConfig.APP_CONFIG_PREFIX_KEY + ":" + appConfig.getAppCode();
		redisTemplate.delete(appCodeKey);

		// 发送redis队列,vertx处理
		// String queue = RedisKeyConfig.VETX_ENVIRONMENT_KEY+vertxEnvironment+":list";
	}

	/***
	 * 加载vertx配置
	 */
	public VertxConfig loadVertxConfig() {
		String vertxConfigKey = RedisKeyConfig.VERTX_CONFIG_STRING_KEY;
		String vertxConfigValue = redisTemplate.opsForValue().get(vertxConfigKey);
		if (StringUtils.isNotBlank(vertxConfigValue)) {
			VertxConfig vertxConfig = JSONObject.parseObject(vertxConfigValue, VertxConfig.class);
			return vertxConfig;
		}
		return null;
	}

	/***
	 * 新增、修改
	 * 
	 * @param appConfig
	 */
	public void addVertxConfig(VertxConfig vertxConfig) {
		String vertxConfigKey = RedisKeyConfig.VERTX_CONFIG_STRING_KEY;
		redisTemplate.opsForValue().set(vertxConfigKey, JSONObject.toJSONString(vertxConfig));

		// 发送redis队列,vertx处理
		// String queue = RedisKeyConfig.VETX_ENVIRONMENT_KEY+vertxEnvironment+":list";
	}
}
